/**
 * apeman bud for ci scripts.
 * @memberof module:apeman-bud-ci/lib
 * @function setup
 * @param {object} params - Template params.
 * @returns {object} - Coz bud configuration.
 */

'use strict'

const argx = require('argx')
const arrayreduce = require('arrayreduce')
const apemanfile = require('apemanfile')
const _bud = require('./_bud')

/** @lends setup */
function setup (params) {
  let args = argx(arguments)
  params = args.shift('object')
  let configuration = apemanfile(params.configuration, {
    only: [
      '$children',
      '$cnsts',
      '$cwd',
      '$pkg',
      '$proto',
      '$parent',
      '$src',
      '$tmpls',
      '$tasks',
      '$notes',
      '$vars'
    ]
  })
  let $children = configuration.$children || {}
  let $tasks = configuration.$tasks || {}
  let bud = Object.keys($tasks).map((name) =>
    _bud(name, params)
  )
  return bud.concat(
    Object.keys($children).map((key) => {
      let $child = $children[ key ]
      let $tasks = $child.$tasks || {}
      let cwd = params.cwd
      return Object.keys($tasks).map((name) => {
        return _bud([ key, name ].join('#'), {
          cwd: cwd ? cwd + '/..' : null
        })
      })
    })
  ).reduce(arrayreduce.arrayConcat(), [])
}

setup.byNames = function (names, params) {
  return names.map((name) => {
    return _bud(name, params || {})
  })
}

module.exports = setup
